import sys
sys.path[0]=("/net/fantasia/home/alextsoi/Software/Python/networkx-1.7")
from networkx import *
from sets import *
from math import *

def unweightedGGGraph(ggfile):
	"""from the input gg/PPI file, return unweighted, undirected graph"""
	uggg=Graph()
	count=0
	for tempstr in open(ggfile):
		if tempstr[0]=="#":
			continue
		count+=1
		if (count%50000)==0:
			print "Processed "+str(count)+" rows..."
		templist=tempstr.rstrip().split("\t")
		uggg.add_edge(templist[0],templist[1])
	return uggg

def unweightedGGsGraph(ggsfile):
	"""from the input ggs file, return unweighted, directed graph"""
	uggsg=DiGraph()
	for tempstr in open(ggsfile):
		if tempstr[0]=="#":
			continue
		templist=tempstr.rstrip().split("\t")
		uggsg.add_edge(templist[0],templist[1])
	return uggsg
	
def weightedGGsGraph_inverse(ggsfile):
	"""from the input ggs file, return weighted, directed graph"""
	ggsg=DiGraph()
	for tempstr in open(ggsfile):
		if tempstr[0]=="#":
			continue
		templist=tempstr.rstrip().split("\t")
		ggsg.add_edge(templist[0],templist[1],weight=1/float(templist[3]))
	return ggsg

def weightedGGsGraph_neg(ggsfile):
	"""from the input ggs file, return weighted, directed graph"""
	ggsg=DiGraph()
	for tempstr in open(ggsfile):
		if tempstr[0]=="#":
			continue
		templist=tempstr.rstrip().split("\t")
		ggsg.add_edge(templist[0],templist[1],weight=(140000-float(templist[3])))
	return ggsg

def density(graph):
	""" return the density of the graph: D=|E|/|E|max"""
	return graph.number_of_edges()/((graph.number_of_nodes()*(float(graph.number_of_nodes())-1))/2)


def nodes_degree(graph, outputf):
	""" return all the nodes degree to output file"""
	outf=open(outputf,'w')
	degree_all=graph.degree(with_labels=True)
	for degree_node in degree_all:
		outf.write(degree_node+"\t"+str(degree_all[degree_node])+"\n")
	outf.close()
	


def shortest_link_all(graph,outputfile):
	""" return all the gene-gene shortest path within the graph in the output file"""
	outf=open(outputfile,'w')
	for node in graph:
		node_shortest_path_length=single_source_shortest_path_length(graph,node)
		outf.write(node+":\t")
		for node_length in node_shortest_path_length:
			outf.write(str(node_shortest_path_length[node_length])+"\t")
		outf.write("\n")
	###for node1 in graph.nodes()[:-1]:
	###	for node2 in graph.nodes()[graph.nodes().index(node1)+1:]:
	###		outf.write(str(shortest_path_length(graph,node1,node2))+"\n")
	outf.close()
	
	
def nodes_clustcoeff(graph, node_s,node_e,outputf):
	""" to get the clustering coefficient for nodes in between node_s and node_e,
	and output to the output file"""
		
	cluscoeffdict=clustering(graph,graph.nodes()[int(node_s):int(node_e)],with_labels=True)
	outf=open(outputf,'w')
	for node in cluscoeffdict:
		outf.write(node+"\t"+str(cluscoeffdict[node])+"\n")
	outf.close()
	
	
def nodes_topologicalCoeff(graph,node_s,node_e,outputf):
	""" for each node (n) in between node_s and node_e, return (to outputf) the list of topological coefficient:
	average of J(n,m),which is the number of shared neighbors 
	between (nodes that sharing at least one node with n) vs (n) [J(n,m) has
	to be +1 if m has direct link with n], divide by the number of neighbors of n
	Nodes that have one or no neighbors are assigned a topological coefficient of 0 (zero)."""
	
	topolist={}
	for node in graph.nodes()[int(node_s):int(node_e)]:
		tempneighborlist=graph.neighbors(node)
		if (len(tempneighborlist)==1) | (len(tempneighborlist)==0):
			topolist[node]=0
			continue
		othernodes=graph.nodes()[:]
		othernodes.remove(node)
		### keeping track of number of other nodes shared at least one partner with node
		num_other_sharepartner=0
		### keeping track of number of partners shared with the other nodes
		num_sharepartner=[]
		for othernode in othernodes:
			othernodeneighborlist=graph.neighbors(othernode)
			temp_num_sharepartner=len(set(tempneighborlist).intersection(set(othernodeneighborlist)))
			if temp_num_sharepartner > 0:
				num_other_sharepartner += 1
				num_sharepartner.append(temp_num_sharepartner+int(othernode in tempneighborlist))
		topolist[node]=(sum(num_sharepartner)/num_other_sharepartner)/float(len(tempneighborlist))
		"""neighborsharednodes=[]
		for tempneighbor in tempneighborlist:
			neighborsharednodes.append(len(set(tempneighborlist).intersection(set(graph.neighbors(tempneighbor)))))
		topolist[node]=sum(neighborsharednodes)/(float(len(tempneighborlist))**2)"""
	outf=open(outputf,'w')
	for node_topo in topolist:
		outf.write(node_topo+"\t"+str(topolist[node_topo])+"\n")
	outf.close()

def Get_dijkstra_shortestLink_oneSource_multipleSink(graph,node_s,outputfD,outputfL,*node_e):
	""" THIS ONLY FOR WEIGHTED GRAPH 
	given the start node (node_s), and a list of ending nodes (node_e), 
	1) output the dijkstra between start and end to outputfD
	2) output the shortest link between start and end to outputfL"""
	
	outD=open(outputfD,"w")
	outL=open(outputfL,"w")
	
	for nodeE in node_e:
		outD.writelines(["%s\t" %item for item in dijkstra_path(graph,node_s,nodeE)])
		outD.write("\n")
		outL.writelines(["%s\t" %item for item in shortest_path(graph,node_s,nodeE)])
		outL.write("\n")
	
	outD.close()
	outL.close()
		
def countEdge_byExp(graph, GGExpfile):
	""" given the graph, count how many of the gene-gene interactions noted
	in GGExpfile are the edges in the graph """
	count=0
	for tempstr in open(GGExpfile):
		tempstr=tempstr.rstrip().split("\t")
		if graph.has_edge(tempstr[0],tempstr[1]):
			count=count+1
	return count

def Get_dijkstra_shortestLink_GGfile(graph,GGExpfile,outputfD,outputfL):
	"""THIS ONLY FOR WEIGHTED GRAPH
	given the weighted graph, output the shortest path distance and the 
	shortest link for each pair of genes in GGExpfile, to the file
	outputfD and outputfL """
	
	outD=open(outputfD,"w")
	outL=open(outputfL,"w")
	
	for tempstr in open(GGExpfile):
		tempstr=tempstr.rstrip().split("\t")
		try:
			if tempstr[0] in graph.nodes() and tempstr[1] in graph.nodes():
				outD.write(tempstr[0]+"\t"+tempstr[1]+"\t"+str(dijkstra_path_length(graph,tempstr[0],tempstr[1]))+"\n")
				outL.write(tempstr[0]+"\t"+tempstr[1]+"\t"+str(shortest_path_length(graph,tempstr[0],tempstr[1]))+"\n")
			else:
				outD.write(tempstr[0]+"\t"+tempstr[1]+"\t"+"NA"+"\n")
				outL.write(tempstr[0]+"\t"+tempstr[1]+"\t"+"NA"+"\n")	
		except Exception:
			outD.write(tempstr[0]+"\t"+tempstr[1]+"\t"+"NA"+"\n")
			outL.write(tempstr[0]+"\t"+tempstr[1]+"\t"+"NA"+"\n")
			
		"""
		if tempstr[0] in graph.nodes() and tempstr[1] in graph.nodes():
			outD.write(tempstr[0]+"\t"+tempstr[1]+"\t"+str(dijkstra_path_length(graph,tempstr[0],tempstr[1]))+"\n")
			outL.write(tempstr[0]+"\t"+tempstr[1]+"\t"+str(shortest_path_length(graph,tempstr[0],tempstr[1]))+"\n")
		else:
			outD.write(tempstr[0]+"\t"+tempstr[1]+"\t"+"NA"+"\n")
			outL.write(tempstr[0]+"\t"+tempstr[1]+"\t"+"NA"+"\n")
		"""
		
	outD.close()
	outL.close()
