"""
kou algorithm:
(i) given a weighted undirected graph and a set of interested nodes
(ii) pair-wise SP between the interested nodes, and keep track of each shortest path's intermediate nodes
(iii) 1st MST algorithm (kruskal's algorithm from networkx)
(iv) expand the tree with intermediate nodes
(v) 2nd MST algorithm (kruskal's algorithm from networkx)
(vi) recurring to trim the non-interested leaves 

e.g.
o=Graph()
o.add_edge(1,2,weight=10)
o.add_edge(1,9,weight=1)
o.add_edge(2,3,weight=8)
o.add_edge(2,6,weight=1)
o.add_edge(3,4,weight=9)
o.add_edge(3,5,weight=2)
o.add_edge(4,5,weight=2)
o.add_edge(5,6,weight=1)
o.add_edge(5,9,weight=1)
o.add_edge(6,7,weight=1)
o.add_edge(7,8,weight=0.5)
o.add_edge(8,9,weight=0.5)

### shortest path cliques: spgraph
spgraph=Graph()
spgraph.add_edge(1,2,weight=4,sp=[1,9,8,7,6,2])
spgraph.add_edge(1,3,weight=4,sp=[1,9,5,3])
spgraph.add_edge(1,4,weight=4,sp=[1,9,5,4])
spgraph.add_edge(2,3,weight=4,sp=[2,6,5,3])
spgraph.add_edge(2,4,weight=4,sp=[2,6,5,4])
spgraph.add_edge(3,4,weight=4,sp=[3,5,4])

(iii) 1st MST algorithm
spgraph_1mst=minimum_spanning_tree(spgraph)


(iv) expand the MST
spgraph_1mst_exp=expMST_Graph(spgraph_1mst,o)

(v) 2nd MST algorithm
spgraph_1mst_exp_2nd=minimum_spanning_tree(spgraph_1mst_exp)

(vi) leaves trimming
spgraph_1mst_exp_2nd_trimmed=trimleaves(spgraph_1mst_exp_2nd,[1,2,3,4])

"""
import sys
import os
sys.path.append("/net/fantasia/home/alextsoi/code/Network")
from ggscoreGraph import *

def expMST_Graph(tree,G):
	### given the MST graph (including the attributes "sp" and "weight"), return a graph expanding all the edges in the shortest path of the tree
	### using the weight from G
	rgraph=Graph()
	for edge in tree.edges(data=True):
		temppath=edge[2]['sp']
		for i in range(len(temppath)-1):
			tempweight=G.get_edge_data(temppath[i],temppath[i+1])['weight']
			rgraph.add_edge(temppath[i],temppath[i+1],weight=tempweight)
	return rgraph

def expMST(tree):
	### given the MST graph (including the attributes "sp" and "weight"), return a graph expanding all the edges in the shortest path of the tree
	### !!! assuming the weight of each edge in sp as 1 (ie PPI)
	rgraph=Graph()
	for edge in tree.edges(data=True):
		temppath=edge[2]['sp']
		#print temppath
		for i in range(len(temppath)-1):
			#print "number of nodes in rgraph: "+str(len(rgraph))
			rgraph.add_edge(temppath[i],temppath[i+1],weight=1)
	return rgraph

def trimleaves(tree,intnodes):
	### given a tree and interested nodes (as a list), recursively trim the non-interested leaves
	newtree=Graph(tree)
	next=True
	while next:
		leaves=(n for n,d in newtree.degree().items() if d==1)
		notintleaves=list(set(leaves)-set(intnodes))
		if len(notintleaves)==0:
			next=False
		else:
			newtree.remove_nodes_from(notintleaves)
	return newtree

def kou_givenclique(spgraph, intnodes):
	### given a SP graph (shortest distance between all nodes, and each edge with the "weight" and "sp" attributes)
	### !!! assuming the weight of each edge in sp as 1 (ie PPI), so use expMST !!!
	###print "start of kou, num of nodes: "+str(len(spgraph))
	### 1st MST algorithm
	spgraph_1mst=Graph(minimum_spanning_tree(spgraph))
	###print "after MST, num of nodes: "+str(len(spgraph_1mst))
	### expand the MST
	spgraph_1mst_exp=Graph(expMST(spgraph_1mst))
	###print "after expansion, num of nodes: "+str(len(spgraph_1mst_exp.nodes()))
	### 2nd MST algorithm
	spgraph_1mst_exp_2nd=Graph(minimum_spanning_tree(spgraph_1mst_exp))
	###print "after MST, num of nodes: "+str(len(spgraph_1mst_exp_2nd.nodes()))
	### leaves trimming
	return Graph(trimleaves(spgraph_1mst_exp_2nd,intnodes))


	
