#! /usr/bin/python
"""
input: 
i) Gene2Fun
ii) SP for function
iii) intrerested function name
iv) Marker2Genes
v) IntMarkers

output:
png graph
"""
from optparse import OptionParser
import sys
import os
from networkx import *
import matplotlib.pyplot as plt
import matplotlib

def main():
	usage="usage: %prog -a genes2fun -s funSP  -p intfun -m marker2gene -i intmarkers -o outprefix"
	parser=OptionParser(usage)
	parser.add_option("-a", "--gene2fun",dest="gene2fun",help="file name of Gene2Fun")
	parser.add_option("-s", "--funSP",dest="funSP",help="file name for the interested function shortest path")
	parser.add_option("-p", "--intfun",dest="intfun",help="name of interested function")
	parser.add_option("-m", "--marker2gene",dest="marker2gene",help="file name of marker2gene")
	parser.add_option("-i", "--intmarkers",dest="intmarkers",help="file name of intmarkers")
	parser.add_option("-o", "--outprefix",dest="outprefix",help="prefix for outputfile")

	(options,args)=parser.parse_args()
	if options.gene2fun==None or options.funSP==None or options.intfun==None or options.marker2gene==None or options.intmarkers==None or options.outprefix==None:
		parser.error("Input arguments are not fully specified")

	### load Genes2Fun
	print("loading Genes2Fun...")
	fun2gene={}
	for tempstr in open(options.gene2fun):
		tempstr=tempstr.rstrip().split()
		if tempstr[1] not in fun2gene:
			fun2gene[tempstr[1]]=[tempstr[0]]
		else:
			fun2gene[tempstr[1]].append(tempstr[0])
	fungenes=fun2gene[options.intfun]
	### load marker2gene
	intmarkers=[]
	for tempstr in open(options.intmarkers):
		tempstr=tempstr.rstrip().split()
		intmarkers.append(tempstr[0])
	intgenes=[]	
	print("loading Marker2Gene...")
	for tempstr in open(options.marker2gene):
		tempstr=tempstr.rstrip().split()
		if tempstr[1] in intmarkers:
			tempgenes=tempstr[4].split(";")
			for g in tempgenes:
				intgenes.append(g)
	### load the funSP
	intfungenes=list(set(fungenes) & set(intgenes))
	G=Graph()
	G.add_nodes_from(intfungenes)
	for tempstr in open(options.funSP):
		if tempstr[0]=="#":
			continue
		tempstr=tempstr.rstrip().split()
		### only continue if both the 2 ends of the shortest path is in the intgenes
		if tempstr[0] not in intgenes or tempstr[1] not in intgenes:
			continue
		temppathgenes=tempstr[3].split(",")
		for i in range(len(temppathgenes)-1):
			G.add_edge(temppathgenes[i],temppathgenes[i+1])

	### after the construction of the graph, identify int nodes and int edges to be colored differently
	nodelist1=intfungenes
	nodelist2=list(set(G.nodes())-set(intfungenes))
	edgelist1=[]
	edgelist2=[]
	for e in G.edges():
		if e[0] in intfungenes and e[1] in intfungenes:
			edgelist1.append(e)
		else:
			edgelist2.append(e)

	### plot
	### pos=shell_layout(G,scale=2)
	pos=spring_layout(G,k=0.1,iterations=50)
	### pos=random_layout(G)
 	plt.axis('off')
 	draw_networkx_nodes(G,pos, nodelist=nodelist1,node_color="r" ,node_size=40,alpha=0.45)
	draw_networkx_nodes(G,pos, nodelist=nodelist2,node_color="b" ,node_size=40,alpha=0.45)
 	
	draw_networkx_edges(G,pos,alpha=0.25,width=0.8,edgelist=edgelist1,edge_color="r")
	draw_networkx_edges(G,pos,alpha=0.25,width=0.8,edgelist=edgelist2,edge_color="b")
 
 	labels={}
	for i in G.nodes():
		labels[i]=i

 	draw_networkx_labels(G,pos,labels,font_size=11) 
	tempname=options.outprefix+".png"
	plt.savefig(tempname,dpi=320)

###################################################################
if __name__ == "__main__":
	main()


